// PROJECT  :MeasuringRPM
// PURPOSE  :Bare bones code shell for IR break-beam strategy DC motor RPM
// COURSE   :ICS3U
// AUTHOR   :C. D'Arcy
// DATE     :2020 04 04
// MCU      :328P
// STATUS   :Working
// REFERENCE:See Library Example: TimerOne > Interrupts
// NOTE     :Use of TimerOne library denies your application's use of pins 9 and 10
#include <TimerOne.h>             //interrupt-based timing support

#define TIMER_INTERVAL 1000000    //# of ms between timer interrupts
#define TEETH_SHIFT 3             //8=2^3 interrupts per rotation (use with RISING or FALLING)             
//#define TEETH_SHIFT 4           //16=2^4 interrupts per rotation (use with CHANGE)

uint16_t RPM;                     //the intended result (rotations per minute)
volatile uint16_t intrCount;      //accumulates # of interrupts per time unit
uint16_t countCopy;               //working with backup offers improved accuracy
volatile uint8_t triggered = 0;   //signals a request for RPM report

void setup() {
  Serial.begin(9600);
  attachInterrupt(digitalPinToInterrupt(2), sense, RISING);
  Timer1.initialize(TIMER_INTERVAL);  //
  Timer1.attachInterrupt(report);     //set function to trigger RPM reporting
}

void sense() {
  intrCount++;                    //increment the # of interrupts per interval
}

void report() {
  triggered = 1;                  //another time interval has elapsed
}

void loop() {
  if (triggered) {
    countCopy = intrCount;        //work with a copy of count for improved accuracy
    intrCount = 0;                //reset count ASAP so as not to miss any...
    triggered = 0;                //prepare for next reporting period
    RPM = (countCopy >> TEETH_SHIFT) * 60;  //convert to rotations per minute 
    Serial.println("RPM: " + String(RPM));  //display 
  }
}
